@@ -1,7 +1,8 @@
-from .utils import get_neutronics_results_from_statepoint_file
+from .utils import process_results
 from .utils import create_initial_particles
 from .utils import extract_points_from_initial_source
 from .utils import silently_remove_file
 from .utils import plotly_trace
+from .utils import diff_between_angles
 
 from .neutronics_model import NeutronicsModel
