@@ -1,11 +1,12 @@
 import os
-import unittest
-from pathlib import Path
 import tarfile
+import unittest
+import urllib.request
+from pathlib import Path
+
 import neutronics_material_maker as nmm
 import openmc
-import openmc_dagmc_wrapper
-import urllib.request
+import openmc_dagmc_wrapper as odw
 
 
 class TestShape(unittest.TestCase):
@@ -14,16 +15,16 @@ class TestShape(unittest.TestCase):
 
     def setUp(self):
 
-        if not Path("tests/v0.0.1.tar.gz").is_file():
-            url = "https://github.com/fusion-energy/neutronics_workflow/archive/refs/tags/v0.0.1.tar.gz"
-            urllib.request.urlretrieve(url, "tests/v0.0.1.tar.gz")
+        if not Path("tests/v0.0.2.tar.gz").is_file():
+            url = "https://github.com/fusion-energy/neutronics_workflow/archive/refs/tags/v0.0.2.tar.gz"
+            urllib.request.urlretrieve(url, "tests/v0.0.2.tar.gz")
 
-            tar = tarfile.open("tests/v0.0.1.tar.gz", "r:gz")
+            tar = tarfile.open("tests/v0.0.2.tar.gz", "r:gz")
             tar.extractall("tests")
             tar.close()
 
-        self.h5m_filename_smaller = "tests/neutronics_workflow-0.0.1/example_01_single_volume_cell_tally/stage_2_output/dagmc.h5m"
-        self.h5m_filename_bigger = "tests/neutronics_workflow-0.0.1/example_02_multi_volume_cell_tally/stage_2_output/dagmc.h5m"
+        self.h5m_filename_smaller = "tests/neutronics_workflow-0.0.2/example_01_single_volume_cell_tally/stage_2_output/dagmc.h5m"
+        self.h5m_filename_bigger = "tests/neutronics_workflow-0.0.2/example_02_multi_volume_cell_tally/stage_2_output/dagmc.h5m"
 
         self.material_description = {
             "tungsten": "tungsten",
@@ -51,17 +52,19 @@ class TestShape(unittest.TestCase):
     def simulation_with_previous_h5m_file(self):
         """This performs a simulation using previously created h5m file"""
 
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "WC"},
         )
 
-        my_model.simulate()
+        my_model.export_xml()
 
-        my_model.process_results()
+        h5m_filename = my_model.simulate()
 
-        my_model.results is not None
+        results = odw.process_results(statepoint_filename=h5m_filename)
+
+        assert results is not None
 
     def test_neutronics_component_simulation_with_openmc_mat(self):
         """Makes a neutronics model and simulates with a cell tally"""
@@ -71,27 +74,31 @@ class TestShape(unittest.TestCase):
         test_mat.set_density(units="g/cm3", density=4.2)
 
         # converts the geometry into a neutronics geometry
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": test_mat},
             cell_tallies=["heating"],
         )
 
-        # performs an openmc simulation on the model
-        output_filename = my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=20,
         )
 
-        assert output_filename.name == "statepoint.2.h5"
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
 
-        results = openmc.StatePoint(output_filename)
+        assert h5m_filename.name == "statepoint.2.h5"
+
+        results = openmc.StatePoint(h5m_filename)
         assert len(results.tallies.items()) == 1
 
-        my_model.process_results(fusion_power=1e9)
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
+            fusion_power=1e9)
         # extracts the heat from the results dictionary
-        assert my_model.results["mat1_heating"]["Watts"]["result"] > 0
+        assert results["mat1_heating"]["Watts"]["result"] > 0
 
     def test_neutronics_component_simulation_with_nmm(self):
         """Makes a neutronics model and simulates with a cell tally"""
@@ -99,25 +106,29 @@ class TestShape(unittest.TestCase):
         test_mat = nmm.Material.from_library("Be")
 
         # converts the geometry into a neutronics geometry
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": test_mat},
             cell_tallies=["heating"],
         )
 
-        # performs an openmc simulation on the model
-        output_filename = my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=20,
         )
 
-        results = openmc.StatePoint(output_filename)
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
+
+        results = openmc.StatePoint(h5m_filename)
         assert len(results.tallies.items()) == 1
 
-        my_model.process_results(fusion_power=1e9)
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
+            fusion_power=1e9)
         # extracts the heat from the results dictionary
-        assert my_model.results["mat1_heating"]["Watts"]["result"] > 0
+        assert results["mat1_heating"]["Watts"]["result"] > 0
 
     # def test_cell_tally_output_file_creation(self):
     #     """Performs a neutronics simulation and checks the cell tally output
@@ -132,7 +143,7 @@ class TestShape(unittest.TestCase):
 
     #     # converts the geometry into a neutronics geometry
     #     # this simulation has no tally to test this edge case
-    #     my_model = openmc_dagmc_wrapper.NeutronicsModel(
+    #     my_model = odw.NeutronicsModel(
     #         h5m_filename=self.h5m_filename_smaller,
     #         source=self.source,
     #         materials={"mat1": test_mat},
@@ -156,34 +167,10 @@ class TestShape(unittest.TestCase):
     #     assert output_filename.name == "statepoint.3.h5"
     #     assert Path("results.json").exists() is True
 
-    def test_incorrect_faceting_tolerance(self):
-        def incorrect_faceting_tolerance():
-            """Sets faceting_tolerance as a string which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
-                h5m_filename=self.h5m_filename_smaller,
-                source=self.source,
-                materials={"mat1": "eurofer"},
-                faceting_tolerance="coucou",
-            )
-
-        self.assertRaises(TypeError, incorrect_faceting_tolerance)
-
-    def test_incorrect_merge_tolerance(self):
-        def incorrect_merge_tolerance():
-            """Set merge_tolerance as a string which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
-                h5m_filename=self.h5m_filename_smaller,
-                source=self.source,
-                materials={"mat1": "eurofer"},
-                merge_tolerance="coucou",
-            )
-
-        self.assertRaises(TypeError, incorrect_merge_tolerance)
-
     def test_incorrect_cell_tallies(self):
         def incorrect_cell_tallies():
             """Set a cell tally that is not accepted which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
@@ -195,7 +182,7 @@ class TestShape(unittest.TestCase):
     def test_incorrect_cell_tally_type(self):
         def incorrect_cell_tally_type():
             """Set a cell tally that is the wrong type which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
@@ -207,7 +194,7 @@ class TestShape(unittest.TestCase):
     def test_incorrect_mesh_tally_2d(self):
         def incorrect_mesh_tally_2d():
             """Set a mesh_tally_2d that is not accepted which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
@@ -219,7 +206,7 @@ class TestShape(unittest.TestCase):
     def test_incorrect_mesh_tally_2d_type(self):
         def incorrect_mesh_tally_2d_type():
             """Set a mesh_tally_2d that is the wrong type which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
@@ -231,7 +218,7 @@ class TestShape(unittest.TestCase):
     def test_incorrect_mesh_tally_3d(self):
         def incorrect_mesh_tally_3d():
             """Set a mesh_tally_3d that is not accepted which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
@@ -243,7 +230,7 @@ class TestShape(unittest.TestCase):
     def test_incorrect_mesh_tally_3d_type(self):
         def incorrect_mesh_tally_3d_type():
             """Set a mesh_tally_3d that is the wrong type which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
@@ -255,7 +242,7 @@ class TestShape(unittest.TestCase):
     def test_incorrect_materials(self):
         def incorrect_materials():
             """Set a material as a string which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials="coucou",
@@ -266,7 +253,7 @@ class TestShape(unittest.TestCase):
     def test_incorrect_materials_type(self):
         def incorrect_materials_type():
             """Sets a material as an int which should raise an error"""
-            test_model = openmc_dagmc_wrapper.NeutronicsModel(
+            test_model = odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": 23},
@@ -279,41 +266,42 @@ class TestShape(unittest.TestCase):
     def test_incorrect_simulation_batches_to_small(self):
         def incorrect_simulation_batches_to_small():
             """Sets simulation batch below 2 which should raise an error"""
-            my_model = openmc_dagmc_wrapper.NeutronicsModel(
+            my_model = odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
             )
-            my_model.simulate(simulation_batches=1)
+            my_model.export_xml(simulation_batches=1)
+            my_model.simulate()
 
         self.assertRaises(ValueError, incorrect_simulation_batches_to_small)
 
     def test_incorrect_simulation_batches_wrong_type(self):
         def incorrect_simulation_batches_wrong_type():
             """Sets simulation_batches as a string which should raise an error"""
-            openmc_dagmc_wrapper.NeutronicsModel(
+            my_model = odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
-                simulation_batches="one",
             )
+            my_model.export_xml(simulation_batches="one")
+            my_model.simulate()
 
         self.assertRaises(TypeError, incorrect_simulation_batches_wrong_type)
 
     def test_incorrect_simulation_particles_per_batch_wrong_type(self):
         def incorrect_simulation_particles_per_batch_wrong_type():
             """Sets simulation_particles_per_batch below 2 which should raise an error"""
-            my_model = openmc_dagmc_wrapper.NeutronicsModel(
+            my_model = odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "eurofer"},
             )
-            my_model.simulate(
-                simulation_particles_per_batch="one",
-            )
+            my_model.export_xml(simulation_batches=1)
+            my_model.simulate()
 
         self.assertRaises(
-            TypeError, incorrect_simulation_particles_per_batch_wrong_type
+            ValueError, incorrect_simulation_particles_per_batch_wrong_type
         )
 
     def test_neutronics_component_cell_simulation_heating(self):
@@ -325,41 +313,39 @@ class TestShape(unittest.TestCase):
         mat.set_density("g/cm3", 2.1)
 
         # converts the geometry into a neutronics geometry
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": mat},
             cell_tallies=["heating", "flux", "TBR", "spectra"],
         )
 
-        # performs an openmc simulation on the model
-        output_filename = my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
-            simulation_particles_per_batch=20,
+            simulation_particles_per_batch=20
         )
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
 
-        results = openmc.StatePoint(output_filename)
+        results = openmc.StatePoint(h5m_filename)
         # spectra add two tallies in this case (photons and neutrons)
         # TBR adds two tallies global TBR and material TBR
         assert len(results.tallies.items()) == 6
         assert len(results.meshes) == 0
 
-        my_model.process_results(fusion_power=1e9)
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
+            fusion_power=1e9
+        )
 
         # extracts the heat from the results dictionary
-        heat = my_model.results["mat1_heating"]["Watts"]["result"]
-        flux = my_model.results["mat1_flux"]["flux per source particle"]["result"]
-        mat_tbr = my_model.results["mat1_TBR"]["result"]
-        tbr = my_model.results["TBR"]["result"]
-        spectra_neutrons = my_model.results["mat1_neutron_spectra"][
-            "flux per source particle"
-        ]["result"]
-        spectra_photons = my_model.results["mat1_photon_spectra"][
-            "flux per source particle"
-        ]["result"]
-        energy = my_model.results["mat1_photon_spectra"]["flux per source particle"][
-            "energy"
-        ]
+        heat = results["mat1_heating"]["Watts"]["result"]
+        flux = results["mat1_flux"]["flux per source particle"]["result"]
+        mat_tbr = results["mat1_TBR"]["result"]
+        tbr = results["TBR"]["result"]
+        spectra_neutrons = results["mat1_neutron_spectra"]["flux per source particle"]["result"]
+        spectra_photons = results["mat1_photon_spectra"]["flux per source particle"]["result"]
+        energy = results["mat1_photon_spectra"]["flux per source particle"]["energy"]
 
         assert heat > 0
         assert flux > 0
@@ -370,6 +356,88 @@ class TestShape(unittest.TestCase):
         assert len(spectra_neutrons) == 709
         assert len(spectra_photons) == 709
 
+    def test_neutronics_spectra_post_processing(self):
+        """Makes a neutronics model and simulates with a cell tally"""
+
+        os.system("rm *.h5")
+        mat = openmc.Material()
+        mat.add_element("Li", 1)
+        mat.set_density("g/cm3", 2.1)
+
+        # converts the geometry into a neutronics geometry
+        my_model = odw.NeutronicsModel(
+            h5m_filename=self.h5m_filename_smaller,
+            source=self.source,
+            materials={"mat1": mat},
+            cell_tallies=["spectra"],
+        )
+
+        my_model.export_xml(
+            simulation_batches=2,
+            simulation_particles_per_batch=2000,
+        )
+
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
+
+        results = odw.process_results(statepoint_filename=h5m_filename)
+        assert len(results.keys()) == 2
+        assert len(results['mat1_neutron_spectra'].keys()) == 1
+        assert len(results['mat1_photon_spectra'].keys()) == 1
+
+        neutron_flux_sum = sum(results['mat1_neutron_spectra']
+                               ["flux per source particle"]["result"])
+        photon_flux_sum = sum(results['mat1_photon_spectra']
+                              ["flux per source particle"]["result"])
+
+        # TODO change > to the actual ratio
+
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
+            fusion_energy_per_pulse=3)
+        assert len(results.keys()) == 2
+        assert len(results['mat1_neutron_spectra'].keys()) == 2
+        assert len(results['mat1_photon_spectra'].keys()) == 2
+        assert sum(results['mat1_neutron_spectra']
+                   ['flux per pulse']['result']) > neutron_flux_sum
+        assert sum(results['mat1_photon_spectra']
+                   ['flux per pulse']['result']) > photon_flux_sum
+
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
+            fusion_power=2)
+        assert len(results.keys()) == 2
+        assert len(results['mat1_neutron_spectra'].keys()) == 2
+        assert len(results['mat1_photon_spectra'].keys()) == 2
+        assert sum(results['mat1_neutron_spectra']
+                   ['flux per second']['result']) > neutron_flux_sum
+        assert sum(results['mat1_photon_spectra']
+                   ['flux per second']['result']) > photon_flux_sum
+
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
+            fusion_energy_per_pulse=2,
+            fusion_power=3)
+        assert len(results.keys()) == 2
+        assert len(results['mat1_neutron_spectra'].keys()) == 3
+        assert len(results['mat1_photon_spectra'].keys()) == 3
+        assert sum(results['mat1_neutron_spectra']
+                   ['flux per pulse']['result']) > neutron_flux_sum
+        assert sum(results['mat1_photon_spectra']
+                   ['flux per pulse']['result']) > photon_flux_sum
+        assert sum(results['mat1_neutron_spectra']
+                   ['flux per second']['result']) > neutron_flux_sum
+        assert sum(results['mat1_photon_spectra']
+                   ['flux per second']['result']) > photon_flux_sum
+
+        spectra_neutrons = results["mat1_neutron_spectra"]["flux per source particle"]["result"]
+        spectra_photons = results["mat1_photon_spectra"]["flux per source particle"]["result"]
+        energy = results["mat1_photon_spectra"]["flux per source particle"]["energy"]
+
+        assert len(energy) == 710
+        assert len(spectra_neutrons) == 709
+        assert len(spectra_photons) == 709
+
     def test_neutronics_component_2d_mesh_simulation(self):
         """Makes a neutronics model and simulates with a 2D mesh tally"""
 
@@ -377,20 +445,21 @@ class TestShape(unittest.TestCase):
         os.system("rm *.h5")
 
         # converts the geometry into a neutronics geometry
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "Be"},
             mesh_tally_2d=["heating"],
         )
 
-        # performs an openmc simulation on the model
-        output_filename = my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=20,
         )
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
 
-        results = openmc.StatePoint(output_filename)
+        results = openmc.StatePoint(h5m_filename)
         assert len(results.meshes) == 3
         assert len(results.tallies.items()) == 3
 
@@ -398,7 +467,7 @@ class TestShape(unittest.TestCase):
         assert Path("heating_on_2D_mesh_xy.png").exists() is False
         assert Path("heating_on_2D_mesh_yz.png").exists() is False
 
-        my_model.process_results(fusion_power=1e9)
+        odw.process_results(statepoint_filename=h5m_filename, fusion_power=1e9)
 
         assert Path("heating_on_2D_mesh_xz.png").exists() is True
         assert Path("heating_on_2D_mesh_xy.png").exists() is True
@@ -411,28 +480,29 @@ class TestShape(unittest.TestCase):
         os.system("rm *.h5 *.vtk")
 
         # converts the geometry into a neutronics geometry
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "Be"},
             mesh_tally_3d=["heating", "(n,Xt)"],
         )
 
-        # performs an openmc simulation on the model
-        output_filename = my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=2,
         )
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
 
-        results = openmc.StatePoint(output_filename)
+        results = openmc.StatePoint(h5m_filename)
         assert len(results.meshes) == 1
         assert len(results.tallies.items()) == 2
-        assert Path(output_filename).exists() is True
+        assert Path(h5m_filename).exists() is True
 
         assert Path("heating_on_3D_mesh.vtk").exists() is False
         assert Path("n-Xt_on_3D_mesh.vtk").exists() is False
 
-        my_model.process_results(fusion_power=1e9)
+        odw.process_results(statepoint_filename=h5m_filename, fusion_power=1e9)
 
         assert Path("heating_on_3D_mesh.vtk").exists() is True
         assert Path("n-Xt_on_3D_mesh.vtk").exists() is True
@@ -446,7 +516,7 @@ class TestShape(unittest.TestCase):
     #     os.system("rm *.h5")
 
     #     # converts the geometry into a neutronics geometry
-    #     my_model = openmc_dagmc_wrapper.NeutronicsModel(
+    #     my_model = odw.NeutronicsModel(
     #         h5m_filename=self.h5m_filename_smaller,
     #         source=self.source,
     #         materials={"mat1": "Be"},
@@ -467,7 +537,7 @@ class TestShape(unittest.TestCase):
         os.system("rm *.h5")
 
         # converts the geometry into a neutronics geometry
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "Be"},
@@ -475,18 +545,20 @@ class TestShape(unittest.TestCase):
             mesh_tally_2d=["heating"],
         )
 
-        # performs an openmc simulation on the model
-        output_filename = my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=2,
         )
-        results = openmc.StatePoint(output_filename)
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
+
+        results = openmc.StatePoint(h5m_filename)
         assert len(results.meshes) == 4  # one 3D and three 2D
         assert len(results.tallies.items()) == 4  # one 3D and three 2D
 
-        my_model.process_results(fusion_power=1e9)
+        odw.process_results(statepoint_filename=h5m_filename, fusion_power=1e9)
 
-        assert Path(output_filename).exists() is True
+        assert Path(h5m_filename).exists() is True
         assert Path("heating_on_3D_mesh.vtk").exists() is True
         assert Path("heating_on_2D_mesh_xz.png").exists() is True
         assert Path("heating_on_2D_mesh_xy.png").exists() is True
@@ -501,7 +573,7 @@ class TestShape(unittest.TestCase):
         os.system("rm *.h5")
 
         # converts the geometry into a neutronics geometry
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "Be"},
@@ -513,18 +585,22 @@ class TestShape(unittest.TestCase):
 
         assert my_model.mesh_3d_corners == [(0, 0, 0), (10, 10, 10)]
         assert my_model.mesh_2d_corners == [(5, 5, 5), (15, 15, 15)]
-        # performs an openmc simulation on the model
-        output_filename = my_model.simulate(
+
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=2,
+
         )
-        results = openmc.StatePoint(output_filename)
+        # performs an openmc simulation on the model
+        h5m_filename = my_model.simulate()
+
+        results = openmc.StatePoint(h5m_filename)
         assert len(results.meshes) == 4  # one 3D and three 2D
         assert len(results.tallies.items()) == 4  # one 3D and three 2D
 
-        my_model.process_results(fusion_power=1e9)
+        odw.process_results(statepoint_filename=h5m_filename, fusion_power=1e9)
 
-        assert Path(output_filename).exists() is True
+        assert Path(h5m_filename).exists() is True
         assert Path("heating_on_3D_mesh.vtk").exists() is True
         assert Path("heating_on_2D_mesh_xz.png").exists() is True
         assert Path("heating_on_2D_mesh_xy.png").exists() is True
@@ -536,7 +612,7 @@ class TestShape(unittest.TestCase):
 
         os.system("rm results.json")
 
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "Be"},
@@ -544,15 +620,20 @@ class TestShape(unittest.TestCase):
             cell_tallies=["TBR", "heating", "flux"],
         )
 
-        # starts the neutronics simulation
-        my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=10,
         )
 
-        my_model.process_results(fusion_power=1e9)
+        # starts the neutronics simulation
+        h5m_filename = my_model.simulate()
 
-        assert isinstance(my_model.results["TBR"]["result"], float)
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
+            fusion_power=1e9
+        )
+
+        assert isinstance(results["TBR"]["result"], float)
         assert Path("results.json").exists() is True
 
     def test_cell_tallies_simulation_fast_flux(self):
@@ -561,7 +642,7 @@ class TestShape(unittest.TestCase):
 
         os.system("rm results.json")
 
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "Be"},
@@ -569,36 +650,39 @@ class TestShape(unittest.TestCase):
             photon_transport=True,
         )
 
-        # starts the neutronics simulation
-        my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=1000,
         )
 
-        my_model.process_results(
+        # starts the neutronics simulation
+        h5m_filename = my_model.simulate()
+
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
             fusion_power=1e9,
             fusion_energy_per_pulse=1.2e6
         )
 
         assert isinstance(
-            my_model.results["mat1_neutron_fast_flux"]["fast flux per source particle"]["result"],
+            results["mat1_neutron_fast_flux"]["fast flux per source particle"]["result"],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_flux"]["flux per source particle"]["result"],
+            results["mat1_flux"]["flux per source particle"]["result"],
             float,
         )
 
-        assert my_model.results["mat1_flux"]["flux per source particle"]["result"] > my_model.results[
+        assert results["mat1_flux"]["flux per source particle"]["result"] > results[
             "mat1_neutron_fast_flux"]["fast flux per source particle"]["result"]
 
     def test_cell_tallies_simulation_effective_dose(self):
         """Performs simulation with h5m file and tallies neutron and photon
         dose. Checks that entries exist in the results."""
 
-        os.system("rm results.json")
+        os.system("rm results.json statepoint*.h5")
 
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             materials={"mat1": "Be"},
@@ -606,88 +690,91 @@ class TestShape(unittest.TestCase):
             photon_transport=True,
         )
 
-        # starts the neutronics simulation
-        my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=10,
         )
 
-        my_model.process_results(
+        # starts the neutronics simulation
+        h5m_filename = my_model.simulate()
+
+        results = odw.process_results(
+            statepoint_filename=h5m_filename,
             fusion_power=1e9,
             fusion_energy_per_pulse=1.2e6
         )
 
         assert isinstance(
-            my_model.results["mat1_neutron_effective_dose"][
+            results["mat1_neutron_effective_dose"][
                 "effective dose per source particle pSv cm3"
             ]["result"],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_neutron_effective_dose"][
+            results["mat1_neutron_effective_dose"][
                 "pSv cm3 per pulse"
             ]["result"],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_neutron_effective_dose"][
+            results["mat1_neutron_effective_dose"][
                 "pSv cm3 per second"
             ]["result"],
             float,
         )
 
         assert isinstance(
-            my_model.results["mat1_neutron_effective_dose"][
+            results["mat1_neutron_effective_dose"][
                 "effective dose per source particle pSv cm3"
             ]["std. dev."],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_neutron_effective_dose"][
+            results["mat1_neutron_effective_dose"][
                 "pSv cm3 per pulse"
             ]["std. dev."],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_neutron_effective_dose"][
+            results["mat1_neutron_effective_dose"][
                 "pSv cm3 per second"
             ]["std. dev."],
             float,
         )
 
         assert isinstance(
-            my_model.results["mat1_photon_effective_dose"][
+            results["mat1_photon_effective_dose"][
                 "effective dose per source particle pSv cm3"
             ]["result"],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_photon_effective_dose"]["pSv cm3 per pulse"][
+            results["mat1_photon_effective_dose"]["pSv cm3 per pulse"][
                 "result"
             ],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_photon_effective_dose"][
+            results["mat1_photon_effective_dose"][
                 "pSv cm3 per second"
             ]["result"],
             float,
         )
 
         assert isinstance(
-            my_model.results["mat1_photon_effective_dose"][
+            results["mat1_photon_effective_dose"][
                 "effective dose per source particle pSv cm3"
             ]["std. dev."],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_photon_effective_dose"]["pSv cm3 per pulse"][
+            results["mat1_photon_effective_dose"]["pSv cm3 per pulse"][
                 "std. dev."
             ],
             float,
         )
         assert isinstance(
-            my_model.results["mat1_photon_effective_dose"][
+            results["mat1_photon_effective_dose"][
                 "pSv cm3 per second"
             ]["std. dev."],
             float,
@@ -701,7 +788,7 @@ class TestShape(unittest.TestCase):
 
         os.system("rm *_on_2D_mesh_*.png")
 
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             h5m_filename=self.h5m_filename_smaller,
             source=self.source,
             # materials=self.material_description,
@@ -709,13 +796,15 @@ class TestShape(unittest.TestCase):
             mesh_tally_2d=["(n,Xt)", "heating", "flux"],
         )
 
-        # starts the neutronics simulation
-        my_model.simulate(
+        my_model.export_xml(
             simulation_batches=2,
             simulation_particles_per_batch=10,
         )
 
-        my_model.process_results(fusion_power=1e9)
+        # starts the neutronics simulation
+        h5m_filename = my_model.simulate()
+
+        odw.process_results(statepoint_filename=h5m_filename, fusion_power=1e9)
 
         assert Path("n-Xt_on_2D_mesh_xz.png").exists() is True
         assert Path("n-Xt_on_2D_mesh_xy.png").exists() is True
@@ -735,15 +824,17 @@ class TestShape(unittest.TestCase):
             """Attempts to simulate without OpenMC xml files which should fail
             with a FileNotFoundError"""
 
-            my_model = openmc_dagmc_wrapper.NeutronicsModel(
+            my_model = odw.NeutronicsModel(
                 h5m_filename=self.h5m_filename_smaller,
                 source=self.source,
                 materials={"mat1": "WC"},
             )
 
+            my_model.export_xml()
+
             os.system("rm *.xml")
 
-            my_model.simulate(export_xml=False)
+            my_model.simulate()
 
         self.assertRaises(
             FileNotFoundError,
@@ -760,7 +851,7 @@ class TestShape(unittest.TestCase):
             import shutil
             shutil.copy(self.h5m_filename_smaller, '.')
 
-            my_model = openmc_dagmc_wrapper.NeutronicsModel(
+            my_model = odw.NeutronicsModel(
                 h5m_filename='dagmc.h5m',
                 source=self.source,
                 materials={"mat1": "WC"},
@@ -783,7 +874,7 @@ class TestShape(unittest.TestCase):
         """Makes a BallReactor neutronics model and simulates the TBR"""
 
         # makes the neutronics material
-        my_model = openmc_dagmc_wrapper.NeutronicsModel(
+        my_model = odw.NeutronicsModel(
             source=openmc.Source(),
             h5m_filename=self.h5m_filename_smaller,
             materials={
